import logging
import re
import subprocess

import MeCab
import pandas as pd

from .tokenizer import Tokenizer, Token

logger = logging.getLogger(__name__)


class Analyzer:
    def __init__(self, tokenizer: Tokenizer, char_filters=[],
                 token_filters=[]):
        """Analyzer Object
        
        Parameters
        ----------
        tokenizer: Tokenizer
            Tokenizer object
        char_filters : list, optional
            CharFilters list, by default []
        token_filters : list, optional
            TokenFilters list, by default []
        """

        self.tokenizer = tokenizer
        self.char_filters = char_filters
        self.token_filters = token_filters

    def analyze(self, text: str):
        """品詞に分解するメソッド.
        
        Parameters
        ----------
        text : str
            解析する文字列
        
        Returns
        -------
        トークンのジェネレーター
        """

        if not text:
            logger.info('text is empty!')
            return
        for cfilter in self.char_filters:
            text = cfilter.filter(text)

        tokens = self.tokenizer.tokenize(text)
        for tfilter in self.token_filters:
            tokens = tfilter.filter(tokens)

        return tokens

    def analyze_with_dataframe(self, text: str):
        """品詞分解したものをpd.DataFrameで返すメソッド
        
        Parameters
        ----------
        text : str
            解析する文字列
        
        Returns
        -------
        pd.DataFrame
        """
        if not text:
            logger.info('text is empty!')
            return

        tokens = list(self.analyze(text))
        columns = list(tokens[0].__dict__.keys())

        dataframe = {}
        for col in columns:
            dataframe[col] = [getattr(token, col) for token in tokens]

        return pd.DataFrame(dataframe)



import os
import logging

name = 'wakame'

PID = os.getpid()
LOGGING_FORMAT = f'[{PID}] %(asctime)s %(levelname)s %(name)s :%(message)s'
logging.basicConfig(level=logging.INFO, format=LOGGING_FORMAT)
logging.addLevelName(logging.ERROR, 'Error')
logging.addLevelName(logging.INFO, 'Info')
logging.addLevelName(logging.WARNING, 'Warning')


import pandas as pd


class TokenFilter:
    def filter(self, tokens):
        return self.apply(tokens)

    def apply(self, tokens):
        raise NotImplementedError


class POSKeepFilter(TokenFilter):
    """ 指定された品詞を残すフィルター """

    def __init__(self, pos_list):
        """
        Parameters
        ----------
        pos_list : list of str
            品詞のリスト.
        """
        self.pos_list = pos_list

    def apply(self, tokens):
        for token in tokens:
            if any(token.part_of_speech.startswith(pos) for pos in self.pos_list):
                yield token


class POSStopFilter(TokenFilter):
    """ 指定された品詞を落とすフィルター """

    def __init__(self, pos_list):
        """
        Parameters
        ----------
        pos_list : list of str
            品詞のリスト.
        """
        self.pos_list = pos_list

    def apply(self, tokens):
        for token in tokens:
            if any(token.part_of_speech.startswith(pos) for pos in self.pos_list):
                continue
            yield token



import re


class CharFilter:
    def filter(self, text):
        return self.apply(text)
    
    def apply(self, text):
        raise NotImplementedError

class RegexReplaceCharFilter(CharFilter):
    def __init__(self, pat, repl):
        """正規表現に一致する部分を置き換える.
        
        Parameters
        ----------
        pat : str
            文字列型の正規表現.
        repl : str
            置き換える文字列.
        """
        self. pattern = re.compile(pat)
        self.replacement = repl
    
    def apply(self, text):
        return re.sub(self.pattern, self.replacement, text)


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='wakame',
    version='0.1.1',
    author='tenajima',
    author_email='tenajima@gmail.com',
    url='https://github.com/tenajima/wakame',
    install_requires=[
        'pandas',
        'mecab-python3',
    ],
    description='janomeライクなインターフェイスを提供するmecabのラッパーです.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Development Status :: 3 - Alpha"
    ]
)


import logging
import re
import subprocess

import MeCab


class Token:
    """
    tokenの情報を内包するクラス
    """

    def __init__(self, word_with_feat: str):
        surface, feature = word_with_feat.split('\t')
        (part_of_speech, infl_type, infl_form, base_form, reading,
         phonetic) = feature.rsplit(',', 5)

        self.surface = surface
        """surface form (表層形)"""
        self.part_of_speech = part_of_speech
        """part of speech (品詞)"""
        self.infl_type = infl_type
        """terminal form (活用型)"""
        self.infl_form = infl_form
        """stem form (活用形)"""
        self.base_form = base_form
        """base form (基本形)"""
        self.reading = reading
        """"reading (読み)"""
        self.phonetic = phonetic
        """pronounce (発音)"""

    def __str__(self):
        return '%s\t%s,%s,%s,%s,%s,%s' % \
           (self.surface,
           self.part_of_speech,
           self.infl_type,
           self.infl_form,
           self.base_form,
           self.reading,
           self.phonetic)


class Tokenizer:
    """
    textの品詞分解を司るクラス
    """

    def __init__(self, use_neologd=False):
        """mecabを使えるようにする
        
        Parameters
        ----------
        use_neologd : bool, optional
            NEologdを使うかどうかを指定する, by default False
        """
        # get neologd
        cmd = 'echo `mecab-config --dicdir`'
        if use_neologd:
            cmd = cmd + '"/mecab-ipadic-neologd"'
        else:
            cmd = cmd + '"/ipadic"'

        # 辞書のパスを取得する
        dict_path = subprocess.Popen(
            cmd, stdout=subprocess.PIPE,
            shell=True).stdout.readline().decode()
        dict_path = re.sub(r'\n', '', dict_path)

        self.tokenizer = MeCab.Tagger('-d ' + dict_path)

    def tokenize(self, text, wakati=False):
        """トークン化する
        
        Parameters
        ----------
        text : str
            品詞分解する対象のtext
        wakati : bool, optional
            Trueにするとsurface部分のみをリストで返すようにする, by default False
        
        Returns
        -------
        トークンのリスト(wakati=False),もしくは文字列のリスト(wakati=True)
        """
        chunk: str = self.tokenizer.parse(text).split('\n')[:-2]
        if wakati:
            tokens = [Token(ch).surface for ch in chunk]
        else:
            tokens = [Token(ch) for ch in chunk]
        return tokens
